@@ -16,6 +16,7 @@ from .utils import (
     get_reproducible_timestamp,
     normalize_archive_path,
     normalize_file_permissions,
+    normalize_inclusion_map,
     replace_file,
     set_zip_info_mode,
 )
@@ -30,15 +31,17 @@ try:
 except ImportError:  # no cov
     EditableProject = None
 
-EDITABLES_MINIMUM_VERSION = '0.2'
+EDITABLES_MINIMUM_VERSION = '0.3'
 
 
 class WheelArchive(object):
-    def __init__(self, metadata_directory, reproducible):
+    def __init__(self, project_id, reproducible):
         """
         https://peps.python.org/pep-0427/#abstract
         """
-        self.metadata_directory = metadata_directory
+        self.metadata_directory = '{}.dist-info'.format(project_id)
+        self.shared_data_directory = '{}.data'.format(project_id)
+
         self.reproducible = reproducible
         if self.reproducible:
             self.time_tuple = self.get_reproducible_time_tuple()
@@ -117,6 +120,16 @@ class WheelArchive(object):
         relative_path = '{}/{}'.format(self.metadata_directory, normalize_archive_path(relative_path))
         return self.write_file(relative_path, contents)
 
+    def add_shared_file(self, shared_file):
+        shared_file.distribution_path = '{}/data/{}'.format(self.shared_data_directory, shared_file.distribution_path)
+        return self.add_file(shared_file)
+
+    def add_extra_metadata_file(self, extra_metadata_file):
+        extra_metadata_file.distribution_path = '{}/extra_metadata/{}'.format(
+            self.metadata_directory, extra_metadata_file.distribution_path
+        )
+        return self.add_file(extra_metadata_file)
+
     def write_file(self, relative_path, contents):
         if not isinstance(contents, bytes):
             contents = contents.encode('utf-8')
@@ -151,6 +164,10 @@ class WheelBuilderConfig(BuilderConfig):
         self.__exclude = []
         self.__packages = []
 
+        self.__core_metadata_constructor = None
+        self.__shared_data = None
+        self.__extra_metadata = None
+
     def set_default_file_selection(self):
         if self.__include or self.__exclude or self.__packages:
             return
@@ -190,6 +207,88 @@ class WheelBuilderConfig(BuilderConfig):
 
         return self.__packages
 
+    @property
+    def core_metadata_constructor(self):
+        if self.__core_metadata_constructor is None:
+            core_metadata_version = self.target_config.get('core-metadata-version', DEFAULT_METADATA_VERSION)
+            if not isinstance(core_metadata_version, str):
+                raise TypeError(
+                    'Field `tool.hatch.build.targets.{}.core-metadata-version` must be a string'.format(
+                        self.plugin_name
+                    )
+                )
+
+            constructors = get_core_metadata_constructors()
+            if core_metadata_version not in constructors:
+                raise ValueError(
+                    'Unknown metadata version `{}` for field `tool.hatch.build.targets.{}.core-metadata-version`. '
+                    'Available: {}'.format(core_metadata_version, self.plugin_name, ', '.join(sorted(constructors)))
+                )
+
+            self.__core_metadata_constructor = constructors[core_metadata_version]
+
+        return self.__core_metadata_constructor
+
+    @property
+    def shared_data(self):
+        if self.__shared_data is None:
+            shared_data = self.target_config.get('shared-data', {})
+            if not isinstance(shared_data, dict):
+                raise TypeError(
+                    'Field `tool.hatch.build.targets.{}.shared-data` must be a mapping'.format(self.plugin_name)
+                )
+
+            for i, (source, relative_path) in enumerate(shared_data.items(), 1):
+                if not source:
+                    raise ValueError(
+                        'Source #{} in field `tool.hatch.build.targets.{}.shared-data` '
+                        'cannot be an empty string'.format(i, self.plugin_name)
+                    )
+                elif not isinstance(relative_path, str):
+                    raise TypeError(
+                        'Path for source `{}` in field `tool.hatch.build.targets.{}.shared-data` '
+                        'must be a string'.format(source, self.plugin_name)
+                    )
+                elif not relative_path:
+                    raise ValueError(
+                        'Path for source `{}` in field `tool.hatch.build.targets.{}.shared-data` '
+                        'cannot be an empty string'.format(source, self.plugin_name)
+                    )
+
+            self.__shared_data = normalize_inclusion_map(shared_data, self.root)
+
+        return self.__shared_data
+
+    @property
+    def extra_metadata(self):
+        if self.__extra_metadata is None:
+            extra_metadata = self.target_config.get('extra-metadata', {})
+            if not isinstance(extra_metadata, dict):
+                raise TypeError(
+                    'Field `tool.hatch.build.targets.{}.extra-metadata` must be a mapping'.format(self.plugin_name)
+                )
+
+            for i, (source, relative_path) in enumerate(extra_metadata.items(), 1):
+                if not source:
+                    raise ValueError(
+                        'Source #{} in field `tool.hatch.build.targets.{}.extra-metadata` '
+                        'cannot be an empty string'.format(i, self.plugin_name)
+                    )
+                elif not isinstance(relative_path, str):
+                    raise TypeError(
+                        'Path for source `{}` in field `tool.hatch.build.targets.{}.extra-metadata` '
+                        'must be a string'.format(source, self.plugin_name)
+                    )
+                elif not relative_path:
+                    raise ValueError(
+                        'Path for source `{}` in field `tool.hatch.build.targets.{}.extra-metadata` '
+                        'cannot be an empty string'.format(source, self.plugin_name)
+                    )
+
+            self.__extra_metadata = normalize_inclusion_map(extra_metadata, self.root)
+
+        return self.__extra_metadata
+
 
 class WheelBuilder(BuilderInterface):
     """
@@ -198,11 +297,6 @@ class WheelBuilder(BuilderInterface):
 
     PLUGIN_NAME = 'wheel'
 
-    def __init__(self, *args, **kwargs):
-        super(WheelBuilder, self).__init__(*args, **kwargs)
-
-        self.__core_metadata_constructor = None
-
     def get_version_api(self):
         return {'standard': self.build_standard, 'editable': self.build_editable}
 
@@ -225,15 +319,14 @@ class WheelBuilder(BuilderInterface):
             else:
                 build_data['tag'] = self.get_default_tag()
 
-        metadata_directory = '{}.dist-info'.format(self.project_id)
-        with WheelArchive(metadata_directory, self.config.reproducible) as archive, closing(StringIO()) as records:
-            for included_file in self.recurse_project_files():
+        with WheelArchive(self.project_id, self.config.reproducible) as archive, closing(StringIO()) as records:
+            for included_file in self.recurse_included_files():
                 record = archive.add_file(included_file)
                 records.write(self.format_record(record))
 
-            self.write_metadata(archive, records, build_data)
+            self.write_data(archive, records, build_data)
 
-            records.write(u'{}/RECORD,,\n'.format(metadata_directory))
+            records.write(u'{}/RECORD,,\n'.format(archive.metadata_directory))
             archive.write_metadata('RECORD', records.getvalue())
 
         target = os.path.join(directory, '{}-{}.whl'.format(self.project_id, build_data['tag']))
@@ -250,16 +343,17 @@ class WheelBuilder(BuilderInterface):
     def build_editable_detection(self, directory, **build_data):
         build_data['tag'] = self.get_default_tag()
 
-        metadata_directory = '{}.dist-info'.format(self.project_id)
-        with WheelArchive(metadata_directory, self.config.reproducible) as archive, closing(StringIO()) as records:
+        with WheelArchive(self.project_id, self.config.reproducible) as archive, closing(StringIO()) as records:
             exposed_packages = {}
-            for included_file in self.recurse_project_files():
+            for included_file in self.recurse_included_files():
                 if not included_file.path.endswith('.py'):
                     continue
 
                 relative_path = included_file.relative_path
-                distribution_path = included_file.distribution_path
+                if not relative_path:
+                    continue
 
+                distribution_path = included_file.distribution_path
                 path_parts = relative_path.split(os.sep)
 
                 # Root file
@@ -278,7 +372,7 @@ class WheelBuilder(BuilderInterface):
                         '{}{}'.format(relative_path[: relative_path.index(distribution_path)], distribution_module),
                     )
 
-            editable_project = EditableProject(self.metadata.core.name.replace('-', '_'), self.root)
+            editable_project = EditableProject(self.metadata.core.name, self.root)
 
             if self.config.dev_mode_exact:
                 for module, relative_path in exposed_packages.items():
@@ -300,9 +394,9 @@ class WheelBuilder(BuilderInterface):
 
                 extra_dependencies.append(dependency)
 
-            self.write_metadata(archive, records, build_data, extra_dependencies=extra_dependencies)
+            self.write_data(archive, records, build_data, extra_dependencies=extra_dependencies)
 
-            records.write(u'{}/RECORD,,\n'.format(metadata_directory))
+            records.write(u'{}/RECORD,,\n'.format(archive.metadata_directory))
             archive.write_metadata('RECORD', records.getvalue())
 
         target = os.path.join(directory, '{}-{}.whl'.format(self.project_id, build_data['tag']))
@@ -313,8 +407,7 @@ class WheelBuilder(BuilderInterface):
     def build_editable_explicit(self, directory, **build_data):
         build_data['tag'] = self.get_default_tag()
 
-        metadata_directory = '{}.dist-info'.format(self.project_id)
-        with WheelArchive(metadata_directory, self.config.reproducible) as archive, closing(StringIO()) as records:
+        with WheelArchive(self.project_id, self.config.reproducible) as archive, closing(StringIO()) as records:
             directories = sorted(
                 os.path.normpath(os.path.join(self.root, relative_directory))
                 for relative_directory in self.config.dev_mode_dirs
@@ -325,9 +418,9 @@ class WheelBuilder(BuilderInterface):
             )
             records.write(self.format_record(record))
 
-            self.write_metadata(archive, records, build_data)
+            self.write_data(archive, records, build_data)
 
-            records.write(u'{}/RECORD,,\n'.format(metadata_directory))
+            records.write(u'{}/RECORD,,\n'.format(archive.metadata_directory))
             archive.write_metadata('RECORD', records.getvalue())
 
         target = os.path.join(directory, '{}-{}.whl'.format(self.project_id, build_data['tag']))
@@ -335,6 +428,17 @@ class WheelBuilder(BuilderInterface):
         replace_file(archive.path, target)
         return target
 
+    def write_data(self, archive, records, build_data, extra_dependencies=()):
+        self.add_shared_data(archive, records)
+
+        # Ensure metadata is written last, see https://peps.python.org/pep-0427/#recommended-archiver-features
+        self.write_metadata(archive, records, build_data, extra_dependencies=extra_dependencies)
+
+    def add_shared_data(self, archive, records):
+        for shared_file in self.recurse_explicit_files(self.config.shared_data):
+            record = archive.add_shared_file(shared_file)
+            records.write(self.format_record(record))
+
     def write_metadata(self, archive, records, build_data, extra_dependencies=()):
         # <<< IMPORTANT >>>
         # Ensure calls are ordered by the number of path components followed by the name of the components
@@ -351,6 +455,9 @@ class WheelBuilder(BuilderInterface):
         # license_files/
         self.add_licenses(archive, records)
 
+        # extra_metadata/ - write last
+        self.add_extra_metadata(archive, records)
+
     def write_archive_metadata(self, archive, records, build_data):
         from packaging.tags import parse_tag
 
@@ -370,7 +477,7 @@ class WheelBuilder(BuilderInterface):
 
     def write_project_metadata(self, archive, records, extra_dependencies=()):
         record = archive.write_metadata(
-            'METADATA', self.core_metadata_constructor(self.metadata, extra_dependencies=extra_dependencies)
+            'METADATA', self.config.core_metadata_constructor(self.metadata, extra_dependencies=extra_dependencies)
         )
         records.write(self.format_record(record))
 
@@ -381,6 +488,11 @@ class WheelBuilder(BuilderInterface):
                 record = archive.write_metadata('license_files/{}'.format(relative_path), f.read())
                 records.write(self.format_record(record))
 
+    def add_extra_metadata(self, archive, records):
+        for extra_metadata_file in self.recurse_explicit_files(self.config.extra_metadata):
+            record = archive.add_extra_metadata_file(extra_metadata_file)
+            records.write(self.format_record(record))
+
     def construct_entry_points_file(self):
         core_metadata = self.metadata.core
         metadata_file = ''
@@ -416,28 +528,6 @@ class WheelBuilder(BuilderInterface):
     def get_default_build_data(self):
         return {'infer_tag': False, 'pure_python': True}
 
-    @property
-    def core_metadata_constructor(self):
-        if self.__core_metadata_constructor is None:
-            core_metadata_version = self.target_config.get('core-metadata-version', DEFAULT_METADATA_VERSION)
-            if not isinstance(core_metadata_version, str):
-                raise TypeError(
-                    'Field `tool.hatch.build.targets.{}.core-metadata-version` must be a string'.format(
-                        self.PLUGIN_NAME
-                    )
-                )
-
-            constructors = get_core_metadata_constructors()
-            if core_metadata_version not in constructors:
-                raise ValueError(
-                    'Unknown metadata version `{}` for field `tool.hatch.build.targets.{}.core-metadata-version`. '
-                    'Available: {}'.format(core_metadata_version, self.PLUGIN_NAME, ', '.join(sorted(constructors)))
-                )
-
-            self.__core_metadata_constructor = constructors[core_metadata_version]
-
-        return self.__core_metadata_constructor
-
     @classmethod
     def get_config_class(cls):
         return WheelBuilderConfig
